from Products.metanav.config import PROJECTNAME, ALLTYPES
from Products.CMFCore.utils import getToolByName

from StringIO import StringIO

def install(self):
    out = StringIO()
    
    # We install our product by running a GS profile.  We use the old-style Install.py module 
    # so that our product works w/ the Quick Installer in Plone 2.5.x
    print >> out, "Installing metanav"
    setup_tool = getToolByName(self, 'portal_setup')
    try:
        # Plone 3.0+
        from Products.CMFPlone.migrations import v3_0
        setup_tool.runAllImportStepsFromProfile('profile-Products.metanav:default')
    except ImportError:
        # Plone 2.5 compatability
        old_context = setup_tool.getImportContextID()
        setup_tool.setImportContext('profile-Products.metanav:default')
        setup_tool.runAllImportSteps()
        setup_tool.setImportContext(old_context)
    print >> out, "Installed types and added to portal_factory via portal_setup"
    
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    
    # Add metanav types to types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f not in typesNotListed:
            typesNotListed.append(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Added metanav types to metaTypesNotToList"
    
    # Add metanav types to types excluded from site search
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for f in ALLTYPES:
        if f not in typesNotSearched:
            typesNotSearched.append(f)
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    print >> out, "Added metanav types to types_not_searched"
    
    # need to do this to add the roles to PAS 
    # and supplement genericsetup until 3.0 comes forth
    uf = getToolByName(self, 'acl_users')
    rmanager = uf.portal_role_manager
    roles = ('NavigationManager',)
    existing = rmanager.listRoleIds()
    for role in roles:
        if role not in existing:
            rmanager.addRole(role)
    print >> out, "Added new role NavigationManager"

    # Install new catalog index(es) using old fashion way,
    # instead of genericsetup catalog.xml so that we
    # don't have to rebuild indexes upon install
    # which may take awhile on a big site.
    catalog_indexes = (
        { 'name'  : 'navURL',
          'type'  : 'FieldIndex'
          },
    )
    catalog_columns = ('navURL',)
    ct = getToolByName(self, 'portal_catalog')
    for idx in catalog_indexes:
        if idx['name'] in ct.indexes():
            out.write("Found the '%s' index in the catalog, nothing changed.\n" % idx['name'])
        else:
            ct.addIndex(**idx)
            out.write("Added '%s' (%s) to the catalog.\n" % (idx['name'], idx['type']))
    for entry in catalog_columns:
        if entry in ct.schema():
            out.write("Found '%s' in the catalog metadatas, nothing changed.\n" % entry)
        else:
            ct.addColumn(entry)
            out.write("Added '%s' to the catalog metadatas.\n" % entry)

    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()
    

def uninstall(self):
    out = StringIO()

    portal_factory = getToolByName(self,'portal_factory')
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    types_tool = getToolByName(self, 'portal_types')
    
    # remove metanav types as a factory type
    factory_types = portal_factory.getFactoryTypes().keys()
    for t in ALLTYPES:
        if t in factory_types:
            factory_types.remove(t)
    portal_factory.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)
    print >> out, "Removed metanav types from portal_factory tool"
    
    # remove metanav types from the portal's list of types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f in typesNotListed:
            typesNotListed.remove(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Removed metanav types from metaTypesNotToList"
    
    # Remove skin directory from skin selections
    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        if 'metanav_content' in path:
            path.remove('metanav_content')
            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)
    print >> out, "Removed metanav_content layer from all skin selections"
    
    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()
    
    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()
