"""metanav -- NavigationItem"""

__author__  = 'Brian Gershon <briang@webcollective.coop>'
__docformat__ = 'plaintext'

try:
    from Products.LinguaPlone.public import *
except ImportError:
    from Products.Archetypes.public import *

## General
from DateTime import DateTime
from AccessControl import ClassSecurityInfo

## Zope
from zope.interface import implements
from zope.event import notify
from Products.CMFCore.utils import getToolByName

## CMF
from Products.CMFCore.permissions import ModifyPortalContent, View

## ATCT + Friends
from Products.ATContentTypes.content.document import ATDocumentSchema, ATDocument
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

##Archetypes
#from Products.Archetypes import atapi

## Product
from Products.metanav.interfaces import INavigationItem
from Products.metanav.interfaces import INavigationItemModifiedEvent
from Products.metanav.config import PROJECTNAME

#V_REQUIRED   = 1
#V_SUFFICIENT = 2

NavigationItemSchema = ATDocumentSchema.copy() + Schema((

    StringField(
        name='summary',
        widget=StringWidget
        (
            label="Summary",
            description="Brief 5-10 word summary",
        ),
    ),

    ReferenceField(
        name='contentpointer',
        widget=ReferenceWidget(
            description="Content this menu item points to",
            label='Points to',
            visible={'view':'visible','edit':'invisible'},
        ),
        relationship="GlobalMenuTo",
        vocabulary_display_path_bound=-1,    #remove long path
    ),

    StringField(
        name='externalurl',
        validators=('isURL'),
        # for testing: validators=(('isEmpty', V_SUFFICIENT), ('isURL', V_REQUIRED)),
        widget=StringWidget(
            label="External URL",
            description="A URL in this field overrides the URL generated from the 'Points to' field above.",
        ),
    ),

))

#NavigationItemSchema['text'].required = False

finalizeATCTSchema(NavigationItemSchema)

class NavigationItemModifiedEvent(object):
    """Event that happens when edits to a Person have been saved"""
    implements(INavigationItemModifiedEvent)

    def __init__(self, context):
        self.context = context

class NavigationItem(ATDocument):
    """A Navigation Item"""
    implements(INavigationItem)
    __implements__ = (getattr(ATDocument,'__implements__',()),)
    
    security = ClassSecurityInfo()
    
    schema         =  NavigationItemSchema
    
    #alter some standard document schema values to hide them from editing
    schema['text'].required = False
    schema['text'].widget.visible = {'view':'invisible','edit':'invisible'}
    schema['relatedItems'].widget.visible = {'view':'invisible','edit':'invisible'}
    schema['description'].widget.visible = {'view':'invisible','edit':'invisible'}
    
    _at_rename_after_creation = True

    suppl_views = ()
    
    typeDescription= """A Navigation Item"""
    
    
    security.declareProtected(View, 'at_post_create_script')
    def at_post_create_script(self):
        """Notify that the Navigation Item has been modified.
        """
        notify(NavigationItemModifiedEvent(self))

    security.declareProtected(View, 'at_post_edit_script')
    def at_post_edit_script(self):
        """Notify that the Navigation Item has been modified.
        """
        notify(NavigationItemModifiedEvent(self))
    
    def navURL(self):
        """
        If externalurl, return that.  Otherwise, get the content
        referenced by our contentpointer field, and return its URL.
        
        Used to populate navURL catalog index.
        """
        utool = getToolByName(self, 'portal_url')
        if self.getExternalurl():
            return self.getExternalurl()
        else:
            refs = self.getRefs(relationship='GlobalMenuTo')
            if len(refs) == 1:
                url = utool.getRelativeContentURL(refs[0])
                return url
            else:
                return None
        

registerATCT(NavigationItem, PROJECTNAME)
