import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.metanav.tests import base
from Products.CMFCore.utils import getToolByName

# zope
from zope.interface.verify import verifyClass
from AccessControl import Unauthorized

# interfaces
from Products.metanav.interfaces import INavigationFolder, INavigationItem, INavigationPortletView
from Products.ATContentTypes.interfaces import IATFolder, IATNewsItem

# content objects
from Products.metanav.content import navigationfolder, navigationitem

from zope.publisher.browser import TestRequest
from zope.component import getMultiAdapter

from zope.component import getUtility
from Products.metanav.interfaces import INavigationFolderVocabulary
from zope.schema.vocabulary import SimpleVocabulary

class testAccessControl(base.MetaNavTestCase):
    """ testing to verify that workflow states and roles are properly granting 
        or denying access
    """

    def afterSetUp(self):
        """ set up basic framework for the security tests.  One portal object, one link
            several users and groups with different roles with respect to the object and 
            link
        """
        self.loginAsPortalOwner()
        # setup users and tools
        self.workflow_tool = getToolByName(self.portal, 'portal_workflow')
        self.portal_catalog = getToolByName(self.portal, 'portal_catalog')
        self.pas = getToolByName(self.portal, 'acl_users')
        self.pas._doAddUser('manager', 'secret', ['Member', 'Manager'], [])
        self.pas._doAddUser('navigationmanager', 'secret', ['Member',  \
            'NavigationManager'], [])
        self.pas._doAddUser('navigationmanager2', 'secret', ['Member', \
            'NavigationManager'], [])
        self.pas._doAddUser('memberuser1', 'secret', ['Member',], [])
        self.pas._doAddUser('memberuser2', 'secret', ['Member',], [])
        self.pas._doAddUser('memberuser3', 'secret', ['Member',], [])
        self.folder.manage_setLocalRoles('memberuser1',['Owner',])
        
        # login as manager
        self.login('manager')
        self.pas._doAddGroup('group1', ['Member',])
        self.g1 = self.pas.getGroupById('group1')
        for m in ('memberuser1','memberuser2'):
            self.g1.addMember(m)
        self.folder.manage_setLocalRoles('group1',['Owner',])
        # memberuser1 is owner of self.folder
        #memberuser2 is not
        # both are in group1 which is owner of self.folder
        
        # create a root folder
        self.portal.portal_types.NavigationRootFolder.global_allow = 1
        self.portal.invokeFactory('NavigationRootFolder', id='navigation_root_folder')
        self.portal.portal_types.NavigationRootFolder.global_allow = 0
        self.rootnavfolder = self.portal.restrictedTraverse('navigation_root_folder')
        self.rootnavfolder.invokeFactory('NavigationFolder', id='nav_folder',
                title="Nav Folder")
        self.nav_folder = self.rootnavfolder.restrictedTraverse('nav_folder')
        
        self.logout()
    
    def _createContentAndLink(self):
        """ login first, then create a piece of content and then request a navlink to it
        """
        # NOTE: form.navigation_location_top_level must match a vocab Title or will err during .update()
        request = TestRequest(form={ 'form.navigation_location_top_level':self.nav_folder.UID(),
                                     'form.navigation_location_top_level_secondary_level':"none",
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.navigation_item_external_url':'',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        
        # create content that has the IATContentType interface
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        self.piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((self.piece_of_content, request), name='navigation_edit_form')
        
        myform.update()
        
        self.nav_link = self.rootnavfolder.nav_folder.restrictedTraverse('my-title')
        
    def _publishNavTree(self):
        """ publish navtree folders
        """
        self.login('manager')
        self.workflow_tool.doActionFor(self.rootnavfolder, 'publish')
        self.workflow_tool.doActionFor(self.nav_folder, 'publish')
        self.logout()
        
    def _getNavTree(self):
        """ get the navtree as the currently logged-in user
        """
        request = TestRequest()
        request.RESPONSE = request.response
        view = getMultiAdapter((self.portal, request), name='navigation_browserview')
        return view.getNavigation()
        
    def testSetupWorked(self):
        """ test that content was created.  Test that link is present and in pending state
            test that member1 owns the link
            test that member2 is owner via group membership
        """
        self.login('memberuser1')
        self._createContentAndLink()
        
        # Did content get created?
        self.failUnless(self.piece_of_content, 'content item is missing')
        self.failUnless(self.nav_link, 'link is missing')

        # Verify piece_of_content and nav_link both are in the workflow states we expect them
        # to start in.
        self.assertEqual(self.workflow_tool.getInfoFor(self.nav_folder, 'review_state'), 'visible',
            "NavigationFolder should have been 'pending'")
        self.assertEqual(self.workflow_tool.getInfoFor(self.nav_link, 'review_state'), 'pending',
            "NavigationItem should have been 'pending'")

        # test that members are in the group        
        self.failUnless(len(self.g1.getGroupMembers())==2, 'There should be 2 members in group1, instead there were only %d' % len(self.g1.getGroupMembers()))
        # test that member1 is an owner of this thing
        m1 = self.pas.getUserById('memberuser1')
        m2 = self.pas.getUserById('memberuser2')
        for item in (self.folder, self.piece_of_content):
            for user in (m1, m2):
                self.failUnless('Owner' in user.getRolesInContext(item), 'user %s is not owner of %s' % (user, item))
        # TODO: group ownership is not being inherited from link target to nav_link so
        #       no need to check this yet.
        self.failUnless('Owner' in m1.getRolesInContext(self.nav_link), 'memberuser1 is not owner of nav_link')
        
    def testAnonymousGetNavigationShouldReturnEmptyListAndNotBlowUp(self):
        """ If nav root is not published, we should see a nice friendly empty dictionary """
        self.logout()
        self.failUnless(not self._getNavTree(), 'This should return [] since nav folder not published')
        
    def testNavLinkAnonymousAccessNavFolder(self):
        """ Publish a link, then test that Anonymous users don't see any links
            for non-published folders, then test to make sure Anonymous can see
            links for published folders. """

        self.login('manager')
        self._createContentAndLink()
        self.workflow_tool.doActionFor(self.rootnavfolder, 'publish')
        self.workflow_tool.doActionFor(self.nav_link, 'publish')

        # First, let's make sure anonymous 
        self.logout()
        self.failUnless(not self._getNavTree(), 'This should return [] since nav folder not published')

        #publish both the folder an a nav item to return in _getNavTree()
        self.login('manager')   # since manager is Owner
        self.workflow_tool.doActionFor(self.nav_folder, 'publish')
        self.logout()
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')

    def testNavLinkAnonymousAccessPublishedLinkOnly(self):
        """ Publish a link, then test that Anonymous users ONLY see PUBLISHED and
            REQUESTREMOVAL links and none that are in Pending. """

        self.login('manager')
        self._createContentAndLink()        
        self._publishNavTree()
        
        # can anonymous see it when pending?
        self.logout()
        nav = self._getNavTree()
        self.failUnless(len(nav[0]['navLevel2'])==0, \
            'Should see nav_folder, but not nav_link since nav_link not published')

        # can anonymous see it when published?
        self.login("navigationmanager")
        self.workflow_tool.doActionFor(self.nav_link, 'publish')
        self.logout()
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')

        # can anonymous see it when pendingremoval?
        self.login("navigationmanager")
        self.workflow_tool.doActionFor(self.nav_link, 'requestremoval')
        self.logout()
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')
    
    def testNavLinkMemberAccess(self):
        """ test that a generic member of the portal can see the link in the navtree 
            when it is published or pendingremoval, but not when it is pending
        """
        self._publishNavTree()
        
        # create the link
        self.login('memberuser1')
        self._createContentAndLink()
        self.logout()
        
        # check if I can see it
        self.login('memberuser3')
        nav = self._getNavTree()
        self.failIf(nav[0]['navLevel2'], 'generic member can see pending nav link')
        self.logout()
        
        # publish the link
        self.login('navigationmanager')
        self.workflow_tool.doActionFor(self.nav_link, 'publish')
        self.logout()
        
        # check if I can see it
        self.login('memberuser3')
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'], 'generic member cannot see published nav link')
        self.logout()
        
        # request removal
        self.login('memberuser1')
        self.workflow_tool.doActionFor(self.nav_link, 'requestremoval')
        self.logout()
        
        # check if I can see it
        self.login('memberuser3')
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'], 'generic member cannot see pendingremoval nav link')
        self.logout()
                
    def testNavLinkOwnerAccess(self):
        """ test that owner of nav_link (the person who requested the link) can see
            the link in the navtree in all three publication states (pending, published,
            and pendingremoval).
        """
        # have memberuser1 create (and own) the content
        self.login('memberuser1')
        self._createContentAndLink()
        self._publishNavTree()
        
        # can owner see it when pending?
        self.login('memberuser1')
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')

        # can owner see it when published?
        self.login("manager")
        self.workflow_tool.doActionFor(self.nav_link, 'publish')
        self.login('memberuser1')
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')

        # can owner see it when pendingremoval?
        self.login("manager")
        self.workflow_tool.doActionFor(self.nav_link, 'requestremoval')
        self.login('memberuser1')
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')
    
    def testNavLinkNavigationManagerAccess(self):
        """ test that NavigationManager can see the link in the navtree in all three publication
            states (pending, published, and pendingremoval)
        """        
        # have memberuser1 create (and own) the content
        self.login('memberuser1')
        self._createContentAndLink()
        self._publishNavTree()
        
        # can navigationmanager see it when pending?
        self.login("navigationmanager")
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')

        # can navigationmanager see it when published?
        self.login("navigationmanager")
        self.workflow_tool.doActionFor(self.nav_link, 'publish')
        self.login('memberuser1')
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')

        # can navigationmanager see it when pendingremoval?
        self.login("navigationmanager")
        self.workflow_tool.doActionFor(self.nav_link, 'requestremoval')
        self.login('memberuser1')
        nav = self._getNavTree()
        self.failUnless(nav[0]['navLevel2'][0]['id'] == 'my-title', 'This should have returned 1 nav link')
    
    def testNavLinkOwnerInGroupAccess(self):
        # this metanav feature not working yet?
        pass
        
if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testAccessControl))
    return suite
        
